c This procedure duplicates the stream of pseudo random numbers generated
c   by the cray INTRINSIC of the same name.  It works on machines which
c   (unlike the cray) follow the IEEE standard for double precision floating
c   point numbers.  Because most machines have 32 bit integers at the largest,
c   the crays 48 bit integer math is duplicated by breaking the problem into
c   steps.  The algorithm is linear congruential.  M is the multiplier and S
c   is the current seed.

      function ranf
C
C NOTE: this version has been modified to use an adder (pn) 
C
      include 'mach.p'
      real*8 ran
      integer*4 M0, M1, M2, M3, Expo, S0, S1, S2, S3
      integer*4 seedhi, seedlo, pn
      integer*4 a, b, c, d, iran(2)
      equivalence(iran, ran)
      common/cseed/seedhi,seedlo,pn
      data M0,M1,M2,M3/0373,3707,1442,0647/

c Split the 48 bit seed into four 12 bit chunks
      S3 = rshift(seedhi,12)
      S2 = and(seedhi,4095)
      S1 = rshift(seedlo,12)
      S0 = and(seedlo,4095)

c IEEE bias is 1023.  Exponent occupies bits 62-52, or 30-20 of the upper word.
      Expo = lshift(1023,20)

c multiply the 12 bit numbers into partial products, discarding terms which
c  are larger than 2^48-1, the max 48 bit integer on the cray.
      a = S0*M0
      b = S1*M0 + S0*M1
      c = S2*M0 + S1*M1 + S0*M2
      d = S3*M0 + S2*M1 + S1*M2 + S0*M3

c assemble the four 26-bit results into two words
      seedlo = a + lshift(and(b,4095),12) + pn
      seedhi = rshift(seedlo,24) + rshift(b,12)+ c + lshift(d,12)
c only want 24 bits in each word (together these make up the 48 bit integer)
      seedlo = and(seedlo,16 777 215)
      seedhi = and(seedhi,16 777 215)

c IEEE mantissa is 52 bits.  We have only 48 bits, so we shift our result 4
c   bits left.  32-(24+4) = 4 bits are still blank in the lower word, so we
c   grab the low 4 bits of seedhi to fill these.  Word is stored with most
c   significant bit at lowest address, so iran(2) is the low word of ran.
      iran(2) = or(lshift(seedlo,4),lshift(seedhi,28))
c place the remaining (24-4)=20 bits of seedhi in bits 20-0 of ran.
c Expo occupies bits 30-20.  Bit 31 (sign) is always zero.
      iran(1) = or(Expo,rshift(seedhi,4))

c The cray's floating point representation is 2^(exp-bias)*0.mantissa.
c The IEEE floating point representation is 2^(exp-bias)*1.mantissa.
c This step subtracts off this extra 1.0, so that we get the same floating point
c    number as on the cray.
      ranf=ran-1.0

      return
      end


