      subroutine intrpa(uk,duk,rknorm,bi,bf,nparts,hbs2m,vk,nspec)
      implicit double precision (a-h,o-z)
c Author: W. Magro.
c Date:   Apr. 26, 1992.
c Description: Numerically integrates the RPA action from beta=bi to beta=bf
c  for nparts(1) protons and nparts(2) electrons with mass given by hbs2m(1)
c  and hbs2m(2) respectively.  The interactions are given by vkpp, vkpe,
c  and vkee.
c  wk is the action associated with the short range potential, given by
c  vspp, vspe, and vsee.
c  The initial values of upp, upe, uee, wpp, wpe, and wee are
c  input in uk(1...6) respectively.  uk also returns the final value.
c  duk returns the final values of the derivatives.
      
      parameter (nmax=6,mnts=500,lptable=mnts-1,pi=3.1415927)
      common /path/ kmax,kount,dxsav,xp(mnts),yp(nmax,mnts)
      common/params/np,ne,rlp,rle,rk,rk2,vkpp,vkpe,vkee
     +              ,vspp,vspe,vsee
      dimension mesh(mnts),splinc(mnts+20,5)
      dimension nparts(3),hbs2m(3),uk(nmax),duk(nmax),vk(nmax)
      external derivs,rkqc
      
      np=nparts(1)
      ne=nparts(2)
      rlp=hbs2m(1)
      rle=hbs2m(2)
      rk=rknorm
      rk2=rk*rk
c this is the long range potential (needed for uk)
      vkpp = vk(1)
      vkpe = vk(2)
      vkee = vk(3)
c this is the short range potential (needed for wk)
      vspp = vk(4)
      vspe = vk(5)
      vsee = vk(6)

      dxsav = (bf-bi)*2
      kmax=500
      nvar=6
c required precision: this value so far is arbitrary
      eps=.000001
c initial stepsize
      h=.01
c minimum acceptable step size
      hmin=1e-9
      call odeint(uk,nvar,bi,bf,eps,h,hmin,nok,nbad,derivs,rkqc)
      call derivs(bf,uk,duk) ! get the final values of the derivatives
      idone=min(kmax,(nok+nbad))-1
      return
      end


      subroutine derivs(x,y,dydx)
      implicit double precision (a-h,o-z)
      common/params/np,ne,rlp,rle,rk,rk2,vkpp,vkpe,vkee
     +              ,vspp,vspe,vsee
c np: Number of protons
c ne: Number of electrons
c rlp: DeBoer parameter for protons
c rle: DeBoer parameter for electrons
c rk:  Norm of k-vector
c rk:  Norm-square of k-vector
c vk:  Long range potential at rk
      parameter (nmax=6)
      dimension y(nmax),dydx(nmax)

c the long range action
      upp=y(1)
      upe=y(2)
      uee=y(3)
c the short range action
      wpp=y(4)
      wpe=y(5)
      wee=y(6)

c the three long range derivatives
      dupp= -(2*rlp*upp + rlp*upp**2*np + rle*upe**2*ne)*(0.5*rk2)
      dupp= dupp - 2*(rlp*wpp*upp*np + rle*wpe*upe*ne)*rk2
      dupp= dupp + vkpp

      dupe= -((rlp+rle)*upe + rlp*upp*upe*np + rle*upe*uee*ne)
     +       *(0.5*rk2)
      !Sums below ensure dupe<->duep symmetry
      dupe= dupe - (rlp*np*(wpp*upe+upp*wpe)+
     +              rle*ne*(wpe*uee+upe*wee))*rk2
      dupe= dupe + vkpe

      duee= -(2*rle*uee + rlp*upe**2*np + rle*uee**2*ne)
     +       *(0.5*rk2)
      duee= duee - 2*(rlp*wpe*upe*np + rle*wee*uee*ne)*rk2
      duee= duee + vkee

c now the short range action
      dwpp= -(2*rlp*wpp + rlp*wpp**2*np + rle*wpe**2*ne)*(0.5*rk2)
      dwpp= dwpp + vspp

      dwpe= -((rlp+rle)*wpe + rlp*wpp*wpe*np + rle*wpe*wee*ne)
     +       *(0.5*rk2)
      dwpe= dwpe + vspe

      dwee= -(2*rle*wee + rlp*wpe**2*np + rle*wee**2*ne)
     +       *(0.5*rk2)
      dwee= dwee + vsee

      dydx(1)=dupp
      dydx(2)=dupe
      dydx(3)=duee
      dydx(4)=dwpp
      dydx(5)=dwpe
      dydx(6)=dwee

      return
      end
